/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: ImageManager.java,v 1.3 2005/08/06 15:42:48 ppoi Exp $
 */
package tsukuba_bunko.peko.canvas.stage;

import java.awt.Image;

import java.awt.image.BufferedImage;

import java.io.IOException;

import java.net.URL;

import javax.imageio.ImageIO;

import tsukuba_bunko.peko.Logger;

import tsukuba_bunko.peko.resource.ResourceManager;


/**
 * 摜f[^Ǘ@\񋟂܂B
 * @author	$Author: ppoi $
 * @version	$Revision: 1.3 $
 */
public class ImageManager	{

	/**
	 * B̃CX^X
	 */
	private static ImageManager	_instance = null;


	/**
	 * <code>ImageManager</code> ̃CX^X쐬܂B
	 */
	protected ImageManager()
	{
		super();
	}


	/**
	 * 摜LbV擾܂BLbVɂȂꍇ͐VKɃ[h܂B
	 * @param	name	摜
	 * @return	摜f[^
	 */
	public Image getImage( String name )
	{
		return getImage( name, false );
	}

	/**
	 * 摜LbV擾܂BLbVɂȂꍇ͐VKɃ[h܂B
	 * @param	name	摜
	 * @return	摜f[^
	 */
	public Image getImage( String name, boolean transparentize )
	{
		BufferedImage	image = null;

		BufferedImage	source = null;
		try	{
			Logger.debug( "[canvas.stage] start to load image \"" + name + "\"." );
			source = ImageIO.read( getURL(name) );
			Logger.debug( "[canvas.stage] finish to load image \"" + name + "\"." );
		}
		catch( IOException ioe )	{
			Logger.debug( "[canvas.stage] fail to load image :" + name, ioe );
			return null;
		}

		if( transparentize )	{
			int	transColor = source.getRGB( 0, 0 );
			if( (transColor & 0xFF000000) != 0 )	{
				image = filterTransparentColor( source, transColor );
				source.flush();
			}
			else	{
				image = source;
			}
		}
		else	{
			image = source;
		}
		return image;
	}

	/**
	 * 摜LbVɖ߂܂B
	 */
	public void putImage( String name, Image image )
	{
		if( image != null )	{
			image.flush();
		}
	}

	/**
	 * 摜 URL ɕϊ܂B
	 * @param	name	摜
	 * @return	摜t@C URL
	 */
	protected URL getURL( String name )
	{
		ResourceManager	resources = ResourceManager.getInstance();
		try	{
			return new URL( resources.getLocationResources().getImagesDirecotryURL(), name );
		}
		catch( Exception e )	{
			Logger.error( "[scene.stage] fatal error!", e );
			return null;
		}
	}

	/**
	 * w肳ꂽF𓧖FƂătB^[V摜𐶐܂B
	 * @param	source	̉摜
	 * @param	transColor	ߐF
	 * @return	ʂ̉摜
	 */
	private BufferedImage filterTransparentColor( BufferedImage source, int transColor )
	{
		Logger.debug( "[canvas.stage] start transparency filtering." );
		int	height = source.getHeight();
		int	width = source.getWidth();
		int	pixel = 0;
		BufferedImage	image = new BufferedImage( width, height, BufferedImage.TYPE_INT_ARGB );
		for( int y = 0; y < height; ++y )	{
			for( int x = 0;x < width; ++x )	{
				pixel = source.getRGB( x, y );
				if( pixel == transColor )	{
					image.setRGB( x, y, 0 );
				}
				else	{
					image.setRGB( x, y , pixel );
				}
			}
		}
		Logger.debug( "[canvas.stage] finish transparency filtering." );
		return image;
	}


	/**
	 * B <code>ImageManager</code> CX^X擾܂B
	 * @return	<code>ImageManager</code> CX^X
	 */
	public static ImageManager getInstance()
	{
		if( _instance == null )	{
			synchronized( ImageManager.class )	{
				if( _instance == null )	{
					_instance = new ImageManager();
				}
			}
		}
		return _instance;
	}
}