/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: SampledAudioClip.java,v 1.3 2005/08/19 03:18:11 ppoi Exp $
 */
package tsukuba_bunko.peko.canvas.stage.audio;

import java.io.IOException;

import java.net.URL;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.FloatControl;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import tsukuba_bunko.peko.Logger;
import tsukuba_bunko.peko.canvas.stage.AudioClip;
import tsukuba_bunko.peko.canvas.stage.AudioPlayer;


/**
 * TEht@CĐ@\񋟂܂B
 * @author	$Author: ppoi $
 * @version	$Revision: 1.3 $ $Date: 2005/08/19 03:18:11 $
 */
public class SampledAudioClip	extends AudioClip	{

	/**
	 * o̓tH[}bg
	 */
	private AudioFormat	_outputFormat = null;

	/**
	 * fR[hςݍĐf[^擾pI[fBI̓Xg[
	 */
	private AudioInputStream	_inputStream = null;

	/**
	 * fR[hOĐf[^擾pI[fBI̓Xg[
	 */
	private AudioInputStream	_baseInputStream = null;

	/**
	 * Đo̓C
	 */
	private SourceDataLine	_line = null;

	/**
	 * }X^[QCRg[
	 */
	private FloatControl	_masterGainControl = null;

	/**
	 * Đp[NXbh
	 */
	private Thread	_workThread = null;

	/**
	 * ĐtO
	 */
	private boolean	_running = false;


	/**
	 * <code>Player</code> ̃CX^X𐶐܂B
	 * ̃RXgN^ō쐬ꂽ Player CX^X̓[vĐ܂B
	 * @param	id	Nbv ID
	 * @param	clipURL	TEht@C URL
	 */
	public SampledAudioClip( String id, URL clipURL )
	{
		super( id, clipURL );
	}


	/**
	 * TEht@CĐ܂B
	 * <p>TEht@C̍Đ͔񓯊ɍsC{\bȟĂяo͂ɕԂ܂B</p>
	 * @throws	UnsupportedAudioFileException	TEht@CT|[g̃I[fBI`̏ꍇ
	 * @throws	IOException	Đ I/O G[ꍇ
	 */
	public void play()
	{
		if( _inputStream != null )	{
			return;
		}

		try {
			setSoundFile( getClipURL() );
		}
		catch( UnsupportedAudioFileException uafe ) {
			//TODO: ObZ[W
			Logger.warn( "unsupported sound format.", uafe );
		}
		catch( LineUnavailableException lue )	{
			//TODO: ObZ[W
			Logger.warn( "source dataLine is not available.", lue );
		}
		catch( IOException ioe ) {
			//TODO: ObZ[W
			Logger.warn( "fail to load sound file.", ioe );
		}

		_workThread = new Thread()	{
			public void run()
			{
				try	{
					AudioInputStream	is = _inputStream;
					SourceDataLine	line = _line;
					byte[]	buffer = new byte[ 8192 ];
					line.start();
					int	readSize = 0;
					while( _running )	{
						readSize = is.read( buffer, 0, buffer.length );
						if( _running )	{
							if( readSize != -1 )	{
								line.write( buffer, 0, readSize );
							}
							else if( isLoop() )	{
								closeInput();
								prepareInput();
								is = _inputStream;
							}
							else	{
								line.drain();
								if( _running )	{
									synchronized( SampledAudioClip.this )	{
										if( _running )	{
											playingFinished();
										}
									}
								}
								break;
							}
						}
					}
				}
				catch( UnsupportedAudioFileException uafe ) {
					//TODO: ObZ[W
					Logger.warn( "unsupported sound format.", uafe );
				}
				catch( IOException ioe ) {
					//TODO: ObZ[W
					Logger.warn( "fail to load sound file.", ioe );
				}
				finally	{
					closeInput();
				}

				synchronized( SampledAudioClip.this )	{
					_workThread = null;
					SampledAudioClip.this.notify();
				}
			}
		};
		_running = true;
		_workThread.start();
	}

	/**
	 * TEht@C̍Đ~܂B
	 * <p>TEht@CĐĂȂꍇCs܂B</p>
	 * @param	mode	~[h
	 */
	public void stop( int mode )
	{
		if( _workThread != null )	{
			synchronized( this )	{
				if( _workThread != null )	{
					playingFinished();
					if( mode == AudioPlayer.STOP_WITH_ASYNC_FADEOUT )	{
						new Thread()	{
							public void run() {
								fadeout();
								_running = false;
							}
						}.start();
					}
					else	{
						if( mode == AudioPlayer.STOP_WITH_SYNC_FADEOUT )	{
							fadeout();
						}

						try {
							_running = false;
							SampledAudioClip.this.wait();
						}
						catch( InterruptedException e ) {
							//ignore
						}
					}
				}
				_workThread = null;
			}
		}
	}

	/**
	 * ̃vC[j܂B
	 */
	public void dispose()
	{
		stop( AudioPlayer.STOP_IMMEDIATELY );
		if( _line != null )	{
			_line.stop();
			_line.close();
		}
		_line = null;
		_masterGainControl = null;
	}

	/**
	 * ĐʂύX܂B
	 * <p>}X^[QCRg[LłȂꍇCs܂B</p>
	 */
	public void setVolume( float level )
	{
		if( _masterGainControl != null )	{
			float	decibel;
			if( level < 0.02f )	{
				decibel = _masterGainControl.getMinimum();
			}
			else	{
				decibel = (float)Math.log( (double)level ) * 20.0f;
			}
			_masterGainControl.setValue( decibel );
		}
	}

	/**
	 * Đʂ擾܂B
	 * <p>}X^[QCRg[LłȂꍇC-1f Ԃ܂B
	 */
	public float getVolume()
	{
		if( _masterGainControl == null )	{
			return -1f;
		}
		else	{
			float	decibel = _masterGainControl.getValue();
			return (float)Math.pow( 10, (decibel / 20.0f) );
		}
	}

	/**
	 * ĐTEht@Cݒ肵CI[fBIVXȅs܂B
	 * @param	clipURL	TEht@C URL
	 * @throws	UnsupportedAudioFileException	TEht@CT|[g̃I[fBI`̏ꍇ
	 * @throws	LineUnavailableException	Đo̓Cgps\̏ꍇ
	 * @throws	IOException	t@C̉͒ I/O G[ꍇ
	 */
	private void setSoundFile( URL clipURL )
		throws UnsupportedAudioFileException, LineUnavailableException, IOException
	{
		prepareInput();

		DataLine.Info	info = new DataLine.Info( SourceDataLine.class, _outputFormat );
		SourceDataLine	line = (SourceDataLine)AudioSystem.getLine( info );
		line.open( _outputFormat );
		_line = line;

		try	{
			_masterGainControl = (FloatControl)line.getControl( FloatControl.Type.MASTER_GAIN );
		}
		catch( IllegalArgumentException iae )	{
			//TODO: ObZ[W
			Logger.warn( "Master Gain Control is not supported.", iae );
		}
	}

	/**
	 * ĐTEht@C܂B
	 * @throws	UnsupportedAudioFileException	TEht@CT|[g̃I[fBI`̏ꍇ
	 * @throws	IOException	͒ I/O G[ꍇ
	 */
	private void prepareInput()
		throws UnsupportedAudioFileException, IOException
	{
		AudioInputStream	baseInputStream = AudioSystem.getAudioInputStream( getClipURL() );
		AudioFormat	baseFormat = baseInputStream.getFormat();
		AudioFormat.Encoding	baseEncoding = baseFormat.getEncoding();
		Logger.debug( "file=" + getClipURL() + " fomart=" + baseFormat + " encoding=" + baseEncoding + "[" + baseEncoding.getClass() + "]" );

		if( AudioFormat.Encoding.class.equals(baseEncoding.getClass()) )	{
			_outputFormat = baseFormat;
			_inputStream = baseInputStream;
			_baseInputStream = null;
		}
		else	{
			_outputFormat = new AudioFormat( AudioFormat.Encoding.PCM_SIGNED, baseFormat.getSampleRate(), 16, baseFormat.getChannels(),
					baseFormat.getChannels() * 2, baseFormat.getSampleRate(), false );
			Logger.debug( "decoded: " + _outputFormat );
			_inputStream = AudioSystem.getAudioInputStream( _outputFormat, baseInputStream );
			_baseInputStream = baseInputStream;
		}
	}

	/**
	 * TEht@CN[Y܂B
	 */
	private void closeInput()
	{
		if( _inputStream != null )	{
			try	{
				_inputStream.close();
			}
			catch( IOException ioe )	{
				//ignore
			}
		}
		if( _baseInputStream != null )	{
			try	{
				_baseInputStream.close();
			}
			catch( IOException ioe )	{
				//ignore
			}
		}
		_inputStream = null;
		_baseInputStream = null;
		_outputFormat = null;
	}

	/**
	 * tF[hAEg܂B
	 */
	private void fadeout()
	{
		synchronized( SampledAudioClip.this )	{
			if( _masterGainControl != null )	{
				float	level = getVolume();
				for( ;  level > 0.03f; level = level * 0.97f )	{//낦ɂ͂ǂ΂悢H
					try	{
						SampledAudioClip.this.wait( 2L );
					}
					catch( InterruptedException ie 	)	{
					}
					setVolume( level );
				}
				_masterGainControl.setValue( _masterGainControl.getMinimum() );
			}
		}
	}
}
