/*
 * All Rights Reserved.
 * Copyright (C) 1999-2005 Tsukuba Bunko.
 *
 * Licensed under the BSD License ("the License"); you may not use
 * this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.tsukuba-bunko.org/licenses/LICENSE.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: AtOnceHandler.java,v 1.2 2005/08/18 14:37:40 ppoi Exp $
 */
package tsukuba_bunko.peko.scenario.stage;

import	org.xml.sax.Attributes;
import	org.xml.sax.SAXException;

import	tsukuba_bunko.peko.scenario.HandlerRegistry;
import	tsukuba_bunko.peko.scenario.PSMLUtil;


/**
 * <samp>at-once</samp> vf ElementHandler łB
 * @author	$Author: ppoi $
 * @version	$Revision: 1.2 $
 */
public class AtOnceHandler	extends StageElementHandler	{

	/**
	 * ݎgp StageElementHandler
	 */
	private StageElementHandler	_child = null;

	/**
	 * level
	 */
	private int	_level = 0;

	/**
	 * XVɎgpGtFNg
	 */
	private String	_effect = null;

	/**
	 * XCh܂ނǂ
	 */
	private boolean	_containsSlideOperation = false;


	/**
	 * <code>AtOnceHandler</code> ̃CX^X쐬܂B
	 */
	public AtOnceHandler()
	{
		super();
	}


//
//	ContentHandler ̎
//
	public void startDocument()
	{
		_child = null;
		_level = 0;
		_containsSlideOperation = false;
		super.startDocument();
	}

	public void endDocument()
	{
		StageCoordinator	coordinator = getStageCoordinator();
		if( _effect == null )	{
			coordinator.updateStage();
		}
		else	{
			if( !coordinator.isSlideVisible() || _containsSlideOperation )	{
				coordinator.updateStage( _effect );
			}
		}
		super.endDocument();
	}

	public void startElement( String namespaceURI, String localName, String qName, Attributes attrs )
		throws SAXException
	{
		if( _child != null )	{
			_level++;
			_child.startElement( namespaceURI, localName, qName, attrs );
		}
		else if( localName.equals("at-once") )	{
			_effect = PSMLUtil.getAttributeValue( attrs, "effect" );
		}
		else	{
			HandlerRegistry	registry = getSceneContext().getSceneProcessor().getHandlerRegistry();
			_child = (StageElementHandler)registry.getElementHandler( namespaceURI, localName );
			if( _child instanceof SlideHandler )	{
				_containsSlideOperation = true;
			}
			if( _child != null )	{
				_child.setParentHandler( this );
				_child.setSceneContext( getSceneContext() );
				_child.startDocument();
				_level++;
				_child.startElement( namespaceURI, localName, qName, attrs );
			}
		}
	}

	public void endElement( String namespaceURI, String localName, String qName )
		throws SAXException
	{
		if( _child != null )	{
			_level--;
			_child.endElement( namespaceURI, localName, qName );
			if( _level == 0 )	{
				_child.endDocument();
				_child.setParentHandler( null );
				_child.setSceneContext( null );
				_child = null;
			}
		}
	}
}
